/*
 * This file is part of Bielefeld Type Library (BTL).
 * 
 * Copyright(c) Frederic Siepmann http://opensource.cit-ec.de/projects/btl
 * 
 * This file may be licensed under the terms of of the GNU Lesser General Public
 * License Version 3 (the ``LGPL''), or (at your option) any later version.
 * 
 * Software distributed under the License is distributed on an ``AS IS'' basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the LGPL for the
 * specific language governing rights and limitations.
 * 
 * You should have received a copy of the LGPL along with this program. If not,
 * go to http://www.gnu.org/licenses/lgpl.html or write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301,
 * USA.
 * 
 * The development of this software was supported by the Excellence Cluster EXC
 * 277 Cognitive Interaction Technology. The Excellence Cluster EXC 277 is a
 * grant of the Deutsche Forschungsgemeinschaft (DFG) in the context of the
 * German Excellence Initiative.
 * 
 * Contributors: Leon Ziegler, Johannes Wienke, Matthias Schröder, Norman
 * Köster, Lukas Twardon, Torben Töniges, Annika Rothert, Marian Pohling, Denis
 * Schulze, Viktor Richter, Birte Carlmeyer, Maikel Linke
 */
package de.unibi.airobots.btl.common;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import nu.xom.Element;
import nu.xom.Elements;
import nu.xom.ParsingException;
import de.unibi.airobots.btl.units.LengthUnit;

/**
 * This is a serializable and iterable polygon type. It is similar to
 * {@link Polygon}, but has a double precision and 3D coordinates.
 * 
 * TODO: implement utility functions like those in {@link java.awt.Polygon}.
 * 
 * @author lziegler
 */
public class PrecisePolygon3D implements Iterable<Point3D> {

	/**
	 * List of polygon points.
	 */
	protected List<Point3D> list = new LinkedList<Point3D>();

	public PrecisePolygon3D(PrecisePolygon3D poly) {
		LengthUnit unit = LengthUnit.MILLIMETER;
		for (Point3D point : poly) {
			list.add(new Point3D(point.getX(unit), point.getY(unit), point
					.getZ(unit), unit));
		}
	}

	public PrecisePolygon3D() {
	}

	/**
	 * Getter for the xml base tag used for this (de-)serialization.
	 * 
	 * @return xml base tag
	 */
	public static String getBaseTag() {
		return "PRECISEPOLYGON3D";
	}

	/**
	 * Constructs a {@link PrecisePolygon3D} object from a given XOM
	 * {@link Element}.
	 * 
	 * @param objectElement
	 *            The XOM {@link Element} to construct an object from.
	 * @return The {@link PrecisePolygon3D} object containing all the
	 *         information given by the {@link Element} object.
	 * @throws ParsingException
	 */
	public static PrecisePolygon3D fromElement(Element objectElement)
			throws ParsingException {

		PrecisePolygon3D poly = new PrecisePolygon3D();

		try {

			if (!objectElement.getLocalName().equals(getBaseTag())) {
				throw new IllegalArgumentException("Parsing Error: Base tag "
						+ "for polygon object must be '" + getBaseTag() + "'");
			}

			Elements points = objectElement.getChildElements(Point3D
					.getBaseTag());
			for (int i = 0; i < points.size(); i++) {
				Element point = points.get(i);
				Point3D p = new Point3D();
				Point3D.fromElement(point, p);
				poly.addPoint(p);
			}

		} catch (NullPointerException ex) {

			// this happens when an element or attribute that is required is
			// not present
			throw new IllegalArgumentException("Missing element or attribute "
					+ "in document.", ex);
		} catch (NumberFormatException e) {
			throw new IllegalArgumentException(
					"could not parse coordinate values");
		}
		return poly;
	}

	public void addPoint(double x, double y, double z, LengthUnit unit) {
		list.add(new Point3D(x, y, z, unit));
	}

	/**
	 * Adds a {@link Point3D} to the polygon.
	 * 
	 * @param p
	 *            The {@link Point3D} instance to add.
	 */
	public void addPoint(Point3D point) {
		list.add(point);
	}

	/**
	 * Serializes the {@link PrecisePolygon3D} object into a given XOM
	 * {@link Element}.
	 * 
	 * @param parent
	 *            The {@link Element} to serialize the object into. The given
	 *            {@link Element} object should have the base tag defined by
	 *            this class. (see {@link #getBaseTag()})
	 * @see #getBaseTag()
	 */
	public void fillInto(Element parent) {
		for (Point3D p : list) {
			Element item = new Element(Point3D.getBaseTag());
			p.fillInto(item);
			parent.appendChild(item);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public String toString() {
		String info = "[PRECISEPOLYGON3D";
		for (Point3D p : list) {
			info += " " + p.toString();
		}
		info += "]";
		return info;
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean equals(Object obj) {
		try {
			if (!(obj instanceof PrecisePolygon3D))
				return false;

			PrecisePolygon3D other = (PrecisePolygon3D) obj;

			return other.list.equals(list);
		} catch (Exception e) {
			return false;
		}
	}

	/**
	 * Returns the centroid of this polygon. Implemented as arithmetic mean
	 * value of all polygon points.
	 * 
	 * @return The centroid of this polygon.
	 */
	public Point3D getCentroid() {

		double x = 0;
		double y = 0;
		double z = 0;
		int count = 0;
		for (Point3D p : list) {
			x += p.getX(LengthUnit.MILLIMETER);
			y += p.getY(LengthUnit.MILLIMETER);
			z += p.getZ(LengthUnit.MILLIMETER);
			count++;
		}

		if (count == 0) {
			return null;
		} else {
			return new Point3D(x / (double) count, y / (double) count, z
					/ (double) count, LengthUnit.MILLIMETER);
		}
	}

	public double getMinX(LengthUnit unit) {
		double minX = Double.POSITIVE_INFINITY;
		for (Point3D point : list) {
			double x = point.getX(unit);
			if (minX > x) {
				minX = x;
			}
		}
		return minX;
	}

	public double getMaxX(LengthUnit unit) {
		double maxX = Double.NEGATIVE_INFINITY;
		for (Point3D point : list) {
			double x = point.getX(unit);
			if (maxX < x) {
				maxX = x;
			}
		}
		return maxX;
	}

	public double getMinY(LengthUnit unit) {
		double minY = Double.POSITIVE_INFINITY;
		for (Point3D point : list) {
			double y = point.getY(unit);
			if (minY > y) {
				minY = y;
			}
		}
		return minY;
	}

	public double getMaxY(LengthUnit unit) {
		double maxY = Double.NEGATIVE_INFINITY;
		for (Point3D point : list) {
			double y = point.getY(unit);
			if (maxY < y) {
				maxY = y;
			}
		}
		return maxY;
	}

	public double getMinZ(LengthUnit unit) {
		double minZ = Double.POSITIVE_INFINITY;
		for (Point3D point : list) {
			double z = point.getZ(unit);
			if (minZ > z) {
				minZ = z;
			}
		}
		return minZ;
	}

	public double getMaxZ(LengthUnit unit) {
		double maxZ = Double.NEGATIVE_INFINITY;
		for (Point3D point : list) {
			double z = point.getZ(unit);
			if (maxZ < z) {
				maxZ = z;
			}
		}
		return maxZ;
	}

	public void move(double dx, double dy, double dz, LengthUnit unit) {
		for (Point3D point : list) {
			point.setX(point.getX(unit) + dx, unit);
			point.setY(point.getY(unit) + dy, unit);
			point.setZ(point.getZ(unit) + dz, unit);
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Iterator<Point3D> iterator() {
		return list.iterator();
	}
}
